print("import: 'betfairlightweight'")
import betfairlightweight

print("import: 'betfairlightweight.endpoints'")
import betfairlightweight.endpoints

print("import: 'betfairlightweight.resources'")
import betfairlightweight.resources

print("import: 'betfairlightweight.streaming'")
import betfairlightweight.streaming

